@@ -23,18 +23,18 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 """
 
-__version__ = (0, 22, 1)
+from doit.version import VERSION
+
+__version__ = VERSION
 
 
-# used to save variable values passed from command line
-CMDLINE_VARS = None
+from doit import loader
+from doit.loader import create_after
+from doit.doit_cmd import get_var
+from doit.api import run
 
-def reset_vars():
-    global CMDLINE_VARS
-    CMDLINE_VARS = {}
+__all__ = ['get_var', 'run', 'create_after']
 
-def get_var(name, default=None):
-    return CMDLINE_VARS.get(name, default)
-
-def set_var(name, value):
-    CMDLINE_VARS[name] = value
+def get_initial_workdir():
+    """working-directory from where the doit command was invoked on shell"""
+    return loader.initial_workdir
