@@ -65,7 +65,7 @@ class List(DoitCmdBase):
                    opt_list_private, opt_list_dependencies, opt_template)
 
 
-    STATUS_MAP = {'ignore': 'I', 'up-to-date': 'U', 'run': 'R'}
+    STATUS_MAP = {'ignore': 'I', 'up-to-date': 'U', 'run': 'R', 'error': 'E'}
 
 
     def _print_task(self, template, task, status, list_deps, tasks):
@@ -77,7 +77,7 @@ class List(DoitCmdBase):
             if self.dep_manager.status_is_ignore(task):
                 task_status = 'ignore'
             else:
-                task_status = self.dep_manager.get_status(task, tasks)
+                task_status = self.dep_manager.get_status(task, tasks).status
             line_data['status'] = self.STATUS_MAP[task_status]
 
         self.outstream.write(template.format(**line_data))
