@@ -6,8 +6,6 @@ import getopt
 import copy
 from collections import OrderedDict
 
-import six
-
 
 
 class DefaultUpdate(dict):
@@ -32,14 +30,14 @@ class DefaultUpdate(dict):
     def add_defaults(self, source):
         """add default values from another dict
         @param source: (dict)"""
-        for key, value in six.iteritems(source):
+        for key, value in source.items():
             if key not in self:
                 self.set_default(key, value)
 
     def update_defaults(self, update_dict):
         """like dict.update but do not update items that have
         a non-default value"""
-        for key, value in six.iteritems(update_dict):
+        for key, value in update_dict.items():
             if key in self._non_default_keys:
                 continue
             self[key] = value
@@ -99,7 +97,7 @@ class CmdOption(object):
         # options can not contain any unrecognized field
         if opt_dict:
             msg = "CmdOption dict contains invalid property '%s'"
-            raise CmdParseError(msg % list(six.iterkeys(opt_dict)))
+            raise CmdParseError(msg % list(opt_dict.keys()))
 
     def __repr__(self):
         tmpl = ("{0}({{'name':{1.name!r}, 'short':{1.short!r}," +
@@ -136,7 +134,7 @@ class CmdOption(object):
         """convert string value to option type value"""
         try:
             # no coversion if value is not a string
-            if not isinstance(str_val, six.string_types):
+            if not isinstance(str_val, str):
                 val = str_val
             elif self.type is bool:
                 val = self.str2boolean(str_val)
@@ -244,7 +242,7 @@ class CmdParse(object):
     def get_short(self):
         """return string with short options for getopt"""
         short_list = ""
-        for opt in six.itervalues(self._options):
+        for opt in self._options.values():
             if not opt.short:
                 continue
             short_list += opt.short
@@ -256,7 +254,7 @@ class CmdParse(object):
     def get_long(self):
         """return list with long options for getopt"""
         long_list = []
-        for opt in six.itervalues(self._options):
+        for opt in self._options.values():
             long_name = opt.long
             if not long_name:
                 continue
@@ -273,7 +271,7 @@ class CmdParse(object):
             - CmdOption from matching opt_str. or None
             - (bool) matched inverse
         """
-        for opt in six.itervalues(self._options):
+        for opt in self._options.values():
             if opt_str in ('-' + opt.short, '--' + opt.long):
                 return opt, False
             if opt_str == '--' + opt.inverse:
@@ -285,7 +283,7 @@ class CmdParse(object):
 
         This values typically come from an INI file
         """
-        for key, val in six.iteritems(new_defaults):
+        for key, val in new_defaults.items():
             if key in self._options:
                 opt = self._options[key]
                 opt.set_default(opt.str2type(val))
@@ -301,7 +299,7 @@ class CmdParse(object):
         """
         params = DefaultUpdate()
         # add default values
-        for opt in six.itervalues(self._options):
+        for opt in self._options.values():
             params.set_default(opt.name, opt.default)
 
         # parse options using getopt
