@@ -4,10 +4,8 @@ import os
 import sys
 import inspect
 import importlib
-import six
 from collections import OrderedDict
 
-from .compat import is_bound_method
 from .exceptions import InvalidTask, InvalidCommand, InvalidDodoFile
 from .task import DelayedLoader, Task, dict_to_task
 
@@ -98,10 +96,16 @@ def get_module(dodo_file, cwd=None, seek_parent=False):
 
 
 
-def create_after(executed=None):
+def create_after(executed=None, target_regex=None, creates=None):
     """Annotate a task-creator function with delayed loader info"""
     def decorated(func):
-        func.doit_create_after = DelayedLoader(func, executed=executed)
+
+        func.doit_create_after = DelayedLoader(
+            func,
+            executed=executed,
+            target_regex=target_regex,
+            creates=creates
+        )
         return func
     return decorated
 
@@ -128,14 +132,27 @@ def load_tasks(namespace, command_names=(), allow_delayed=False):
     # from different modules
     funcs.sort(key=lambda obj: obj[2])
 
-    task_list = []
-    for name, ref, _ in funcs:
-        delayed = (allow_delayed and getattr(ref, 'doit_create_after', None))
 
-        if delayed:
-            task_list.append(Task(name, None, loader=delayed))
-        else:
-            task_list.extend(generate_tasks(name, ref(), ref.__doc__))
+    task_list = []
+    def _process_gen():
+        task_list.extend(generate_tasks(name, ref(), ref.__doc__))
+    def _add_delayed(tname):
+        task_list.append(Task(tname, None, loader=delayed,
+                              doc=delayed.creator.__doc__))
+
+    for name, ref, _ in funcs:
+        delayed = getattr(ref, 'doit_create_after', None)
+
+        if not delayed:  # not a delayed task, just run creator
+            _process_gen()
+        elif delayed.creates:  # delayed with explicit task basename
+            for tname in delayed.creates:
+                _add_delayed(tname)
+        elif allow_delayed:  # delayed no explicit name, cmd run
+            _add_delayed(name)
+        else:  # delayed no explicit name, cmd list (run creator)
+            _process_gen()
+
     return task_list
 
 
@@ -151,7 +168,7 @@ def _get_task_creators(namespace, command_names):
     funcs = []
     prefix_len = len(TASK_STRING)
     # get all functions that are task-creators
-    for name, ref in six.iteritems(namespace):
+    for name, ref in namespace.items():
 
         # function is a task creator because of its name
         if ((inspect.isfunction(ref) or inspect.ismethod(ref)) and
@@ -165,13 +182,13 @@ def _get_task_creators(namespace, command_names):
             # If create_doit_tasks is a method, it should be called only
             # if it is bounded to an object.
             # This avoids calling it for the class definition.
-            argspec = inspect.getargspec(ref)
-            if len(argspec.args) != (1 if is_bound_method(ref) else 0):
+            if inspect.signature(ref).parameters:
                 continue
             task_name = name
 
         # ignore functions that are not a task creator
-        elif True: # coverage can't get "else: continue"
+        else:  # pragma: no cover
+            # coverage can't get "else: continue"
             continue
 
         # tasks can't have the same name of a commands
@@ -300,7 +317,7 @@ def generate_tasks(func_name, gen_result, gen_doc=None):
                 _generate_task_from_yield(tasks, func_name, task_dict, x_doc)
 
         if tasks:
-            return list(six.itervalues(tasks))
+            return list(tasks.values())
         else:
             # special case task_generator did not generate any task
             # create an empty group task
