@@ -1,12 +1,6 @@
 import os
 import time
-import multiprocessing
-import six
-if six.PY3: # pragma: no cover
-    from dbm import whichdb
-else:
-    from whichdb import whichdb
-
+from dbm import whichdb
 
 import py
 import pytest
@@ -20,20 +14,28 @@ def get_abspath(relativePath):
     return os.path.join(os.path.dirname(__file__), relativePath)
 
 # fixture to create a sample file to be used as file_dep
-@pytest.fixture
-def dependency1(request):
-    path = get_abspath("data/dependency1")
-    if os.path.exists(path): os.remove(path)
-    ff = open(path, "w")
-    ff.write("whatever" + str(time.asctime()))
-    ff.close()
+def dependency_factory(relative_path):
 
-    def remove_dependency():
-        if os.path.exists(path):
-            os.remove(path)
-    request.addfinalizer(remove_dependency)
+    @pytest.fixture
+    def dependency(request):
+        path = get_abspath(relative_path)
+        if os.path.exists(path): os.remove(path)
+        ff = open(path, "w")
+        ff.write("whatever" + str(time.asctime()))
+        ff.close()
 
-    return path
+        def remove_dependency():
+            if os.path.exists(path):
+                os.remove(path)
+        request.addfinalizer(remove_dependency)
+
+        return path
+
+    return dependency
+
+
+dependency1 = dependency_factory("data/dependency1")
+dependency2 = dependency_factory("data/dependency2")
 
 # fixture to create a sample file to be used as file_dep
 @pytest.fixture
@@ -144,6 +146,14 @@ def tasks_sample():
     return tasks_sample
 
 
+def tasks_bad_sample():
+    """Create list of tasks that cause errors."""
+    bad_sample = [
+        Task("e1", [""], doc='e4 bad file dep', file_dep=['xxxx'])
+    ]
+    return bad_sample
+
+
 def CmdFactory(cls, outstream=None, task_loader=None, dep_file=None,
                backend=None, task_list=None, sel_tasks=None,
                dep_manager=None, config=None, cmds=None):
@@ -161,33 +171,3 @@ def CmdFactory(cls, outstream=None, task_loader=None, dep_file=None,
     cmd.task_list = task_list  # list of tasks
     cmd.sel_tasks = sel_tasks  # from command line or default_tasks
     return cmd
-
-
-# mokey patch multiprocessing to enable  code coverage
-# NOTE: doesnt work with pytest-xdist (actually execnet)
-def coverage_multiprocessing_process(): # pragma: no cover
-    try:
-        import coverage as _coverage
-        _coverage
-    except:
-        return
-
-    from coverage.collector import Collector
-    from coverage.control import coverage
-    # detect if coverage was running in forked process
-    if Collector._collectors:
-        original = multiprocessing.Process._bootstrap
-        class Process_WithCoverage(multiprocessing.Process):
-            def _bootstrap(self):
-                cov = coverage(data_suffix=True)
-                cov.start()
-                try:
-                    return original(self)
-                finally:
-                    cov.stop()
-                    cov.save()
-        return Process_WithCoverage
-
-ProcessCoverage = coverage_multiprocessing_process()
-if ProcessCoverage:
-    multiprocessing.Process = ProcessCoverage
