@@ -1,13 +1,22 @@
 import time
 from multiprocessing import Process
 
+import pytest
+
 from doit.cmdparse import DefaultUpdate
 from doit.task import Task
 from doit.cmd_base import TaskLoader
+from doit import filewatch
 from doit import cmd_auto
 from .conftest import CmdFactory
 
 
+# skip all tests in this module if platform not supported
+platform = filewatch.get_platform_system()
+pytestmark = pytest.mark.skipif(
+    'platform not in filewatch.FileModifyWatcher.supported_platforms')
+
+
 class TestFindFileDeps(object):
     def find_deps(self, sel_tasks):
         tasks = {
@@ -51,6 +60,32 @@ class TestAuto(object):
         assert cmd.parse_execute(['t2']) == 3
 
 
+    def test_run_callback(self, monkeypatch):
+        result = []
+        def mock_cmd(callback, shell=None):
+            result.append(callback)
+        monkeypatch.setattr(cmd_auto, 'call', mock_cmd)
+
+        # success
+        result = []
+        cmd_auto.Auto._run_callback(0, 'success', 'failure')
+        assert 'success' == result[0]
+
+        # failure
+        result = []
+        cmd_auto.Auto._run_callback(3, 'success', 'failure')
+        assert 'failure' == result[0]
+
+        # nothing executed
+        result = []
+        cmd_auto.Auto._run_callback(0, None , None)
+        assert 0 == len(result)
+        cmd_auto.Auto._run_callback(1, None , None)
+        assert 0 == len(result)
+
+
+
+
     def test_run_wait(self, dependency1, target1, depfile_name):
         def ok():
             with open(target1, 'w') as fp:
