@@ -1,8 +1,6 @@
-# coding=UTF-8
-
 import sys
 import json
-from six import StringIO
+from io import StringIO
 
 from doit import reporter
 from doit.task import Task
@@ -57,6 +55,11 @@ class TestConsoleReporter(object):
         assert "-- " in rep.outstream.getvalue()
         assert "t_name" in rep.outstream.getvalue()
 
+    def test_skipUptodate_hidden(self):
+        rep = reporter.ConsoleReporter(StringIO(), {})
+        rep.skip_uptodate(Task("_name", None))
+        assert "" == rep.outstream.getvalue()
+
     def test_skipIgnore(self):
         rep = reporter.ConsoleReporter(StringIO(), {})
         rep.skip_ignore(Task("t_name", None))
